/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actionwithtargetdialog.h"

#include <qobjectlist.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>

#include <kdebug.h>

using namespace SubtitleComposer;

ActionWithTargetDialog::ActionWithTargetDialog( const QString& title, QWidget* parent ):
	ActionDialog( title, parent ),
	m_targetButtonGroup( 0 ),
	m_targetLayout( 0 ),
	m_linesTargetButtonGroup( 0 ),
	m_textsTargetButtonGroup( 0 ),
	m_selectionTargetOnlyMode( false ),
	m_selectionTargetWasChecked( false ),
	m_translationMode( false ),
	m_nonTranslationModeTarget( SubtitleLine::Primary )
{
}

ActionWithTargetDialog::~ActionWithTargetDialog()
{
	delete m_linesTargetButtonGroup;
	delete m_textsTargetButtonGroup;
}

int ActionWithTargetDialog::exec()
{
	return ActionDialog::exec();
}

QButtonGroup* ActionWithTargetDialog::createTargetButtonGroup( const QString& title, bool addToLayout )
{
	if ( ! m_targetButtonGroup )
	{
		m_targetButtonGroup = createButtonGroup( title, addToLayout );
		m_targetLayout = createLayout( m_targetButtonGroup );
	}

	return m_targetButtonGroup;
}

void ActionWithTargetDialog::updateTargetGroupHidden()
{
	bool hidden = true;
	QObjectList objects( *m_targetButtonGroup->children() );
	for ( QObject* object = objects.first(); object; object = objects.next() )
	{
		if ( object->isWidgetType() )
		{
			if ( ! static_cast<QWidget*>( object )->isHidden() )
			{
				hidden = false;
				break;
			}
		}
	}

	if ( hidden == m_targetButtonGroup->isHidden() )
		return;

	if ( hidden )
		m_targetButtonGroup->hide();
	else
		m_targetButtonGroup->show();
}

void ActionWithTargetDialog::setTargetButtonsHidden( QButtonGroup* targetButtonGroup, int buttonGroupColumn, bool hidden )
{
	if ( ! targetButtonGroup || hidden == targetButtonGroup->find( 0 )->isHidden() )
		return;

	int index = 0;
	if ( hidden )
	{
		for ( QButton* button = targetButtonGroup->find( index ); button != 0; button = targetButtonGroup->find( ++index ) )
			button->hide();
		m_targetLayout->setColStretch( buttonGroupColumn, 0 );
	}
	else
	{
		for ( QButton* button = targetButtonGroup->find( index ); button != 0; button = targetButtonGroup->find( ++index ) )
			button->show();
		m_targetLayout->setColStretch( buttonGroupColumn, 1 );
	}

	updateTargetGroupHidden();
}



/// LINES TARGET
/// ============

void ActionWithTargetDialog::createLinesTargetButtons()
{
	createTargetButtonGroup();

	m_linesTargetButtonGroup = new QButtonGroup( 0 );
	m_linesTargetButtonGroup->hide();

	for ( int index = 0; index < None; ++index )
	{
		QRadioButton* radioButton = new QRadioButton( m_targetButtonGroup );
		m_linesTargetButtonGroup->insert( radioButton, index );
		m_targetLayout->addWidget( radioButton, index, 0 );
	}

	m_targetLayout->setColStretch( 0, 1 );

	m_linesTargetButtonGroup->find( AllLines )->setText( i18n( "All lines" ) );
	m_linesTargetButtonGroup->find( Selection )->setText( i18n( "Selected lines" ) );
	m_linesTargetButtonGroup->find( FromSelected )->setText( i18n( "All lines from first selected" ) );
	m_linesTargetButtonGroup->find( UpToSelected )->setText( i18n( "All lines up to last selected" ) );

	static_cast<QRadioButton*>( m_linesTargetButtonGroup->find( AllLines ) )->setChecked( true );

	_setSelectionTargetOnlyMode( m_selectionTargetOnlyMode, true );
}

ActionWithTargetDialog::LinesTarget ActionWithTargetDialog::selectedLinesTarget() const
{
	if ( ! m_linesTargetButtonGroup ) // lines target was not created
		return AllLines;

	int checkedId = m_linesTargetButtonGroup->selectedId();
	return checkedId == -1 ? None : (LinesTarget)checkedId;
}

void ActionWithTargetDialog::setSelectedLinesTarget( ActionWithTargetDialog::LinesTarget target )
{
	if ( m_linesTargetButtonGroup && m_linesTargetButtonGroup->find( target ) )
		static_cast<QRadioButton*>( m_linesTargetButtonGroup->find( target ) )->setChecked( true );
}

bool ActionWithTargetDialog::isLinesTargetEnabled( LinesTarget target ) const
{
	return	m_linesTargetButtonGroup &&
			m_linesTargetButtonGroup->find( target ) &&
			m_linesTargetButtonGroup->find( target )->isEnabled();
}

void ActionWithTargetDialog::setLinesTargetEnabled( LinesTarget target, bool enabled )
{
	if ( m_linesTargetButtonGroup && m_linesTargetButtonGroup->find( target ) )
		m_linesTargetButtonGroup->find( target )->setEnabled( enabled );
}

bool ActionWithTargetDialog::selectionTargetOnlyMode() const
{
	return m_selectionTargetOnlyMode;
}

void ActionWithTargetDialog::setSelectionTargetOnlyMode( bool value )
{
	_setSelectionTargetOnlyMode( value, false );
}

void ActionWithTargetDialog::_setSelectionTargetOnlyMode( bool value, bool force )
{
	if ( force || m_selectionTargetOnlyMode != value )
	{
		m_selectionTargetOnlyMode = value;

		if ( ! m_targetButtonGroup || ! m_linesTargetButtonGroup )
			return;

		if ( m_selectionTargetOnlyMode )
		{
			m_selectionTargetWasChecked = static_cast<QRadioButton*>( m_linesTargetButtonGroup->find( Selection ) )->isChecked();
			m_linesTargetButtonGroup->find( Selection )->setEnabled( true );
			static_cast<QRadioButton*>( m_linesTargetButtonGroup->find( Selection ) )->setChecked( true );
		}
		else
		{
			if ( ! m_selectionTargetWasChecked )
			{
				m_linesTargetButtonGroup->find( AllLines )->setEnabled( true );
				static_cast<QRadioButton*>( m_linesTargetButtonGroup->find( AllLines ) )->setChecked( true );
			}
		}

		setTargetButtonsHidden( m_linesTargetButtonGroup, 0, m_selectionTargetOnlyMode );
	}
}



/// TEXTS TARGET
/// ============

void ActionWithTargetDialog::createTextsTargetButtons()
{
	createTargetButtonGroup();

	m_textsTargetButtonGroup = new QButtonGroup( 0 );
	m_textsTargetButtonGroup->hide();

	for ( int index = 0; index < SubtitleLine::OpModeUNKNOWN; ++index )
	{
		QRadioButton* radioButton = new QRadioButton( m_targetButtonGroup );
		m_textsTargetButtonGroup->insert( radioButton, index );
		m_targetLayout->addWidget( radioButton, index, 1 );
	}

	m_targetLayout->setColStretch( 1, 1 );

	m_textsTargetButtonGroup->find( SubtitleLine::Both )->setText( i18n( "Both subtitles" ) );
	m_textsTargetButtonGroup->find( SubtitleLine::Primary )->setText( i18n( "Primary subtitle" ) );
	m_textsTargetButtonGroup->find( SubtitleLine::Secondary )->setText( i18n( "Translation subtitle" ) );

	_setTranslationMode( m_translationMode, true );
}

SubtitleLine::OpMode ActionWithTargetDialog::nonTranslationModeTarget() const
{
	return m_nonTranslationModeTarget;
}

void ActionWithTargetDialog::setNonTranslationModeTarget( SubtitleLine::OpMode target )
{
	if ( m_nonTranslationModeTarget != target )
	{
		if ( ! m_translationMode && m_textsTargetButtonGroup )
		{
			static_cast<QRadioButton*>( m_textsTargetButtonGroup->find( m_nonTranslationModeTarget ) )->setChecked( false );
			static_cast<QRadioButton*>( m_textsTargetButtonGroup->find( target ) )->setChecked( true );
		}

		m_nonTranslationModeTarget = target;
	}
}

SubtitleLine::OpMode ActionWithTargetDialog::selectedTextsTarget() const
{
	if ( ! m_textsTargetButtonGroup ) // texts target was not created
		return SubtitleLine::OpModeUNKNOWN;

	int checkedId = m_textsTargetButtonGroup->selectedId();
	return checkedId == -1 ? SubtitleLine::OpModeUNKNOWN : (SubtitleLine::OpMode)checkedId;
}

void ActionWithTargetDialog::setSelectedTextsTarget( SubtitleLine::OpMode target )
{
	if ( m_textsTargetButtonGroup && m_textsTargetButtonGroup->find( target ) )
		static_cast<QRadioButton*>( m_textsTargetButtonGroup->find( target ) )->setChecked( true );
}

bool ActionWithTargetDialog::isTextsTargetEnabled( SubtitleLine::OpMode target ) const
{
	return	m_textsTargetButtonGroup &&
			m_textsTargetButtonGroup->find( target ) &&
			m_textsTargetButtonGroup->find( target )->isEnabled();
}

void ActionWithTargetDialog::setTextsTargetEnabled( SubtitleLine::OpMode target, bool enabled )
{
	if ( m_textsTargetButtonGroup && m_textsTargetButtonGroup->find( target ) )
		m_textsTargetButtonGroup->find( target )->setEnabled( enabled );
}

bool ActionWithTargetDialog::translationMode() const
{
	return m_translationMode;
}

void ActionWithTargetDialog::setTranslationMode( bool enabled )
{
	_setTranslationMode( enabled, false );
}

void ActionWithTargetDialog::_setTranslationMode( bool enabled, bool force )
{
	if ( force || m_translationMode != enabled )
	{
		m_translationMode = enabled;

		if ( ! m_targetButtonGroup || ! m_textsTargetButtonGroup )
			return;

		if ( ! m_translationMode )
			static_cast<QRadioButton*>( m_textsTargetButtonGroup->find( m_nonTranslationModeTarget ) )->setChecked( true );

		setTargetButtonsHidden( m_textsTargetButtonGroup, 1, ! m_translationMode );
	}
}



/// ACTION WITH LINES TARGET DIALOG
/// ===============================

ActionWithLinesTargetDialog::ActionWithLinesTargetDialog( const QString& title, QWidget* parent ):
	ActionWithTargetDialog( title, parent )
{
	createLinesTargetButtons();
}

ActionWithLinesTargetDialog::ActionWithLinesTargetDialog( const QString& title, const QString& desc, QWidget* parent ):
	ActionWithTargetDialog( title, parent )
{
	createTargetButtonGroup( desc );
	createLinesTargetButtons();
}

int ActionWithLinesTargetDialog::exec()
{
	return m_targetButtonGroup->isHidden() ? QDialog::Accepted : ActionWithTargetDialog::exec();
}



/// ACTION WITH TEXTS TARGET DIALOG
/// ===============================

ActionWithTextsTargetDialog::ActionWithTextsTargetDialog( const QString& title, QWidget* parent ):
	ActionWithTargetDialog( title, parent )
{
	createTextsTargetButtons();
}

ActionWithTextsTargetDialog::ActionWithTextsTargetDialog( const QString& title, const QString& desc, QWidget* parent ):
	ActionWithTargetDialog( title, parent )
{
	createTargetButtonGroup( desc );
	createTextsTargetButtons();
}

int ActionWithTextsTargetDialog::exec()
{
	return m_targetButtonGroup->isHidden() ? QDialog::Accepted : ActionWithTargetDialog::exec();
}



/// ACTION WITH LINES AND TEXTS TARGET DIALOG
/// =========================================

ActionWithLinesAndTextsTargetDialog::ActionWithLinesAndTextsTargetDialog( const QString& title, QWidget* parent ):
	ActionWithTargetDialog( title, parent )
{
	createLinesTargetButtons();
	createTextsTargetButtons();
}

ActionWithLinesAndTextsTargetDialog::ActionWithLinesAndTextsTargetDialog( const QString& title, const QString& desc, QWidget* parent ):
	ActionWithTargetDialog( title, parent )
{
	createTargetButtonGroup( desc );
	createLinesTargetButtons();
	createTextsTargetButtons();
}

int ActionWithLinesAndTextsTargetDialog::exec()
{
	return m_targetButtonGroup->isHidden() ? QDialog::Accepted : ActionWithTargetDialog::exec();
}
